import java.util.Iterator;

/**
 * BinaryTree.java defines the interface to a binary tree data structure
 */
public interface BinaryTree<E> {


   /**
     * Should be implemented to return true if the binary tree is
     *  empty and false otherwise.
     */
   public boolean isEmpty();

   /**  
     * Should be implemented to return the number of elements in the
     *  binary tree.
     */
   public int size();
   
	/**
    * Should be implemented to return the root of the binary tree.
    */
	public BinaryNode<E> getRoot();
   
 	/**
  	  * Should be implemented to return a new BinaryTree whose root
  	  * is referenced to the left/right subtree.
  	  * 
  	  * Notice that this binary tree and its left/right subtree have
  	  * to be updated consistently.  The new subtrees are not detached.
  	  */
 	public BinaryTree<E> getLeftBtree();
 	public BinaryTree<E> getRightBtree();
 	
 	/**
 	 * Should be implemented to return a reference to the element
 	 * if it is found in the root, null otherwise.
     */
 	public E getDataBtree();


   /**
     * Should be implemented to perform an inorder traversal on the
     * binary tree by calling an overloaded, recursive inorder method
     * that starts with the root.
     */
    public Iterator<E> iteratorInOrder();


   /**
     * Should be implemented to perform an preorder traversal on the
     * binary tree by calling an overloaded, recursive preorder method
     * that starts with the root.
     */
    public Iterator<E> iteratorPreOrder();

   
   /**
     * Should be implemented to perform a postorder traversal on the
     * binary tree by calling an overloaded, recursive postorder method
     * that starts with the root.
     */
    public Iterator<E> iteratorPostOrder();

   
   /**
     * Should be implemented to perform a levelorder traversal on
     * the binary tree, using a queue.
     */
    public Iterator<E> iteratorLevelOrder();
   

}  // interface BinaryTreeADT


