import java.util.NoSuchElementException;


/**
 * An implementation of the stack interface using singly-linked nodes. 
 *
 */
public class LinkedStack<T> implements MyStack<T> {

   /**
   * A Node<T> provides a generic singly-linked node for a linked list. Each node 
   * contains a piece of data (which is a reference to an T object) and a link
   * (which is a reference to the next node of the list). The reference stored
   * in a node can be null.
   */
   private class Node {
       public T data;
       public Node next;

      /**
      * Initialize a node with a specified initial data and link to the next
      * node. Note that the initialLink may be the null reference, 
      * which indicates that the new node has nothing after it.
      *
      * @param data the initial data of this new node
      * @param next a reference to the node after this new node--this reference may be null
      *   to indicate that there is no node after this new node.
      */ 
       public Node(T data, Node next) {
            this.data = data;
            this.next = next;
       }
    } //end class Node


    private Node top = null;


    /**
     * Adds the given item to the top of the stack.
     *
     * @param item the item to be pushed onto this stack.
     */
    public void push(T item) {
        top = new Node(item, top);
    }

    /**
     * Removes the top item from the stack and returns it.
     *	
     * @return the item at the top of this stack.
     * @exception java.util.NoSuchElementException if the stack is empty.
     */
    public T pop() {
	if (top == null) {
	    throw new NoSuchElementException("Cannot pop from empty stack");
	}
        T item = top.data;
        top = top.next;
        return item;
    }

    /**
     * Returns whether the stack is empty or not.
     *
     * @return true if this stack contains no items, false otherwise.
     */
    public boolean isEmpty() {
        return top == null;
    }

    /**
     * Returns the top item from the stack without popping it.
     *
     * @return the object at the top of this stack. 
     * @exception java.util.NoSuchElementException if the stack is empty.
     */
    public T peek() {
        if (top == null) {
            throw new NoSuchElementException("Cannot peek into empty stack");
        }
        return top.data;
    }

    /**
     * Returns the number of items currently in the stack.
     *
     * @return the number of elements in this stack.
     */
    public int size() {
        int count = 0;
        for (Node node = top; node != null; node = node.next) {
            count++;
        }
        return count;
    }
}